@@ -7,6 +7,6 @@ Copyright (c) 2021 Roi ZHAO
 
 '''
 
-from .rquote import get_price, get_stock_concepts, get_concept_stks
+from .rquote import get_price, get_stock_concepts, get_concept_stocks
 from .rquote import get_all_concepts, get_all_industries
 from .utils import CommonUtils, WebUtils, BasicFactors, DataFormatter, reqget
