@@ -21,7 +21,7 @@ def make_tgts(mkts=['ch', 'hk', 'us', 'fund', 'future'], money_min=2e8) -> []:
     cands = []
 
 
-def get_cn_stocks_by_amount(money_min=2e8):
+def get_cn_stock_list(money_min=2e8):
     '''
     Return sorted stock list ordered by latest amount of money, cut at `money_min`
     item in returned list are [code, name, change, amount, mktcap]
@@ -53,42 +53,57 @@ def get_cn_stocks_by_amount(money_min=2e8):
     return a
 
 
-def get_hk_stocks_hotest80():
-    a = reqget(
-        'http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php' +
-        '/Market_Center.getHKStockData?page=1&num=80&sort=amount&asc=0&node=' +
-        'qbgg_hk&_s_r_a=sort').text
-    if a:
-        a = [['hk'+i['symbol'], i['name'], i['changepercent'], i['amount'],
-            i['market_value']] for i in json.loads(a)]
-    return a
+# def get_hk_stocks_hotest80():
+#     a = reqget(
+#         'http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php' +
+#         '/Market_Center.getHKStockData?page=1&num=80&sort=amount&asc=0&node=' +
+#         'qbgg_hk&_s_r_a=sort').text
+#     if a:
+#         a = [['hk'+i['symbol'], i['name'], i['changepercent'], i['amount'],
+#             i['market_value']] for i in json.loads(a)]
+#     return a
 
 
-def get_us_stocks_hotest30():
+def get_us_stocks_biggest(k=60):
+    # return list of [symbol, name, price, volume, mktcap, pe]
+    uscands = []
     a = reqget(
-        'https://xueqiu.com/service/v5/stock/screener/quote/list?page=1&' +
-        'size=30&order=desc&orderby=amount&order_by=amount&market=US&type=us&_=' +
-        str(int(time.time() * 1000)), headers=WebUtils.headers()).text
+        "https://stock.finance.sina.com.cn/usstock/api/jsonp.php/IO.XSRV2."+
+        "CallbackList['f0j3ltzVzdo2Fo4p']/US_CategoryService.getList?page=1"+
+        "&num=60&sort=&asc=0&market=&id=", headers=WebUtils.headers()).text
     if a:
-        uslist = json.loads(a)['data']['list']
+        uslist = json.loads(a.split('(')[1][:-2])['data']
         # Warning: symbol not fitted
-        uscands = ['us' + i['symbol'] + '.OQ' for i in uslist]
+        uscands = [('us' + i['symbol'], i['name'], i['price'], i['volume'],
+            i['mktcap']) for i in uslist]
     return uscands
 
 
-def get_cn_fund():
+def get_cn_fund_list():
+    '''
+    Return sorted etf list (ordered by latest amount of money),
+        of [code, name, change, amount, price]
+    '''
     a = reqget(base64.b64decode('aHR0cDovL3ZpcC5zdG9jay5maW5hbmNlLnNpbmEuY29tL'+
         'mNuL3F1b3Rlc19zZXJ2aWNlL2FwaS9qc29ucC5waHAvSU8uWFNSVjIuQ2FsbGJhY2tMaX'+
         'N0WydrMldhekswNk5Rd2xoeVh2J10vTWFya2V0X0NlbnRlci5nZXRIUU5vZGVEYXRhU2l'+
         'tcGxlP3BhZ2U9MSZudW09MTAwMCZzb3J0PWFtb3VudCZhc2M9MCZub2RlPWV0Zl9ocV9m'+
         'dW5kJiU1Qm9iamVjdCUyMEhUTUxEaXZFbGVtZW50JTVEPXhtNGkw').decode()).text
     if a:
-        fundcands = [[i['symbol'],i['name'],i['changepercent']*100,i['amount'],i['trade']]
+        fundcands = [[i['symbol'], i['name'], i['changepercent'], i['amount'], i['trade']]
                      for i in json.loads(a.split('k2WazK06NQwlhyXv')[1][3:-2])]
     return fundcands
 
 
-def get_cn_future():
+def get_cn_future_list():
+    '''
+    Return cn future id list, with prefix of `fu`
+    e.g. ['fuSC2109',
+          'fuRB2110',
+          'fuHC2110',
+          'fuFU2109',
+          ...]
+    '''
     a = reqget('https://finance.sina.com.cn/futuremarket/').text
     if a:
         futurelist_active = [
@@ -122,24 +137,20 @@ def get_price(i, sdate='', edate='', freq='day', days=320, fq='hfq',
     qtimg_stock_us = base64.b64decode('aHR0cDovL3dlYi5pZnpxLmd0aW1nLmNuL2FwcHN' +
                                       '0b2NrL2FwcC91c2Zxa2xpbmUvZ2V0P3BhcmFtPX' +
                                       't9LHt9LHt9LHt9LHt9LGhmcQ==').decode('utf-8')
-    sina_future_d = base64.b64decode('aHR0cHM6Ly9zdG9jazIuZmluYW5jZS5zaW5hLmNv' +
-                                     'bS5jbi9mdXR1cmVzL2FwaS9qc29ucC5waHAvdmFy' +
-                                     'JTIwX3t9e309L0lubmVyRnV0dXJlc05ld1NlcnZp' +
-                                     'Y2UuZ2V0RGFpbHlLTGluZT9zeW1ib2w9e30mXz17fQ==').decode()
-    # sina_future_d.format('FB0','2020_11_14','FB0','2020_11_14')
+    sina_future_d = 'https://stock2.finance.sina.com.cn/futures/api/jsonp.php/' + \
+            'var%20t1nf_{}=/InnerFuturesNewService.getDailyKLine?symbol={}'
+    # sina_future_d.format('FB0','FB0')
 
     if i[:2] == 'BK':
         try:
             a = reqget(base64.b64decode('aHR0cDovL3B1c2gyaGlzLmVhc3' +
-                                                   'Rtb25leS5jb20vYXBpL3F0L3N0' +
-                                                   'b2NrL2tsaW5lL2dldD9jYj1qUX' +
-                                                   'VlcnkxMTI0MDIyNTY2NDQ1ODcz' +
-                                                   'NzY2OTcyXzE2MTc4NjQ1NjgxMz' +
-                                                   'Emc2VjaWQ9OTAu').decode() + i +
-                                  '&fields1=f1%2Cf2%2Cf3%2Cf4%2Cf5' +
-                                  '&fields2=f51%2Cf52%2Cf53%2Cf54%2Cf55%2Cf56%2Cf57%2Cf58' +
-                                  '&klt=101&fqt=0&beg=19900101&end=20220101&_=1',
-                                  headers=WebUtils.headers())
+                               'Rtb25leS5jb20vYXBpL3F0L3N0b2NrL2tsaW5lL2dldD9jYj1qUX' +
+                               'VlcnkxMTI0MDIyNTY2NDQ1ODczNzY2OTcyXzE2MTc4NjQ1NjgxMz' +
+                               'Emc2VjaWQ9OTAu').decode() + i +
+                          '&fields1=f1%2Cf2%2Cf3%2Cf4%2Cf5' +
+                          '&fields2=f51%2Cf52%2Cf53%2Cf54%2Cf55%2Cf56%2Cf57%2Cf58' +
+                          '&klt=101&fqt=0&beg=19900101&end=20220101&_=1',
+                          headers=WebUtils.headers())
             if not a:
                 logging.warning('{} reqget failed: {}'.format(i, a))
                 return i, 'None', pd.DataFrame([])
@@ -160,12 +171,13 @@ def get_price(i, sdate='', edate='', freq='day', days=320, fq='hfq',
 
     if i[:2] == 'fu':
         try:
+            ix = i[2:] if i[-1]=='0' else i[2:-4]
             d = pd.DataFrame(json.loads(reqget(sina_future_d.format(
-                i[2:-4] + '0', '', i[2:-4] + '0', '')).text.split('(')[1][:-2]))
-            d.columns = ['date', 'open', 'close', 'high', 'low', 'vol', 'p']
+                    ix, ix)).text.split('(')[1][:-2]))
+            d.columns = ['date', 'open', 'high', 'low', 'close', 'vol', 'p', 's']
             d = d.set_index(['date']).astype(float)
             # d.index = pd.DatetimeIndex(d.index)
-            return i[2:-4], '', d
+            return i, '', d
         except Exception as e:
             logging.warning('error get price {}, err {}'.format(i[2:-4], e))
             return i, 'None', pd.DataFrame([])
@@ -215,9 +227,41 @@ def get_price_longer(i, l=2, dd={}):
     return i, name, a
 
 
+def get_tick(tgts=[]):
+    '''
+    Get quotes of a tick
+    tgt list format:
+        us stocks like gb_symbol, e.g. gb_aapl, gb_goog
+    Return list of dict of given symbols for current timestamp
+    '''
+    if not tgts:
+        return []
+    sina_tick = 'https://hq.sinajs.cn/?list='
+    head_row = ['name', 'price', 'price_change_rate', 'timesec',
+        'price_change', '_', '_', '_', '_', '_', 'volume', '_', '_',
+         '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_',
+         '_', 'last_close', '_', '_', '_', 'turnover', '_', '_', '_', '_']
+
+    if type(tgts) == list:
+        tgts = ['gb_' + i[2:].lower() for i in tgts]
+    elif type(tgts) == str:
+        tgts = ['gb_' + tgts]
+    else:
+        raise ValueError('tgt should be list or str with market prefix, e.g. usAPPL,')
+    txt = reqget(sina_tick + ','.join(tgts))
+    if not txt:
+        logging.warning('reqget failed {}'.format(tgts))
+        return []
+
+    dat = [i.split('"')[1].split(',') for i in txt.split(';\n') if '"' in i]
+    dat_trim = [{k:i[j] for j,k in enumerate(head_row) if k!='_'} for i in dat]
+    #logging.warming('data not complete: {}'.format(tgts))
+    return dat_trim
+
+
 def get_stock_concepts(i) -> []:
     '''
-    Return concept list of a stock, from eastmoney
+    Return concept id(start with `BK`) list of a stock, from eastmoney
     '''
     f10url = base64.b64decode('aHR0cDovL2YxMC5lYXN0bW9uZXkuY29tLy9Db3JlQ29uY2V' +
                               'wdGlvbi9Db3JlQ29uY2VwdGlvbkFqYXg/Y29kZT0=').decode()
@@ -236,7 +280,7 @@ def get_stock_concepts(i) -> []:
     return concepts
 
 
-def get_concept_stks(bkid, dc=None):
+def get_concept_stocks(bkid, dc=None):
     '''
     Return stocks of input bkid, e.g. BK0420, BK0900
     dc : dictionary of concepts, local cache with get/put
@@ -263,6 +307,7 @@ def get_concept_stks(bkid, dc=None):
 
 def _east_list_fmt(burl, api_name):
     '''
+    formatter of eastmoney api
     Return list of list:
         [sid, name, rise, amount, mkt]
     '''
@@ -278,6 +323,10 @@ def _east_list_fmt(burl, api_name):
 
 
 def get_all_industries():
+    '''
+    Return sorted industry item list ordered by latest amount of money,
+    item in returned list are [code, name, change, amount, price]
+    '''
     a = _east_list_fmt('aHR0cHM6Ly84Ny5wdXNoMi5lYXN0bW9uZXkuY29tL2FwaS9xdC9jbGl'+
         'zdC9nZXQ/Y2I9alF1ZXJ5MTEyNDAzNzExNzU2NTU3MTk3MTM0NV8xNjI3MDQ3MTg4NTk5'+
         'JnBuPTEmcHo9MTAwJnBvPTEmbnA9MSZ1dD1iZDFkOWRkYjA0MDg5NzAwY2Y5YzI3ZjZmN'+
@@ -289,6 +338,10 @@ def get_all_industries():
 
 
 def get_all_concepts():
+    '''
+    Return sorted concept item list ordered by latest amount of money,
+    item in returned list are [code, name, change, amount, price]
+    '''
     a = _east_list_fmt('aHR0cHM6Ly8yMi5wdXNoMi5lYXN0bW9uZXkuY29tL2FwaS9xdC9jbGl'+
         'zdC9nZXQ/Y2I9alF1ZXJ5MTEyNDA3MzI5ODQxOTMwNzY4OTc5XzE2MjcxMDk0NjA2MzMm'+
         'cG49MSZwej00MDAmcG89MSZucD0xJnV0PWJkMWQ5ZGRiMDQwODk3MDBjZjljMjdmNmY3N'+
@@ -300,6 +353,10 @@ def get_all_concepts():
 
 
 def get_bk_stocks(bkid):
+    '''
+    Return stock item list of given bk id,
+    item in returned list are [code, name, change, amount, price]
+    '''
     url = base64.b64decode('aHR0cDovLzgyLnB1c2gyLmVhc3Rtb25leS5jb20vYXBpL3F0L2'+
         'NsaXN0L2dldD9jYj1qUXVlcnkxMTI0MDQ4Njk5NjMwMDk1MTM3NzE0XzE2Mjc0Nzc0OTU'+
         'wNjQmcG49MSZwej0yMDAwJnBvPTAmbnA9MSZ1dD1iZDFkOWRkYjA0MDg5NzAwY2Y5YzI3'+
@@ -312,6 +369,10 @@ def get_bk_stocks(bkid):
 
 
 def get_industry_stocks(bkid):
+    '''
+    Return sorted industry item list ordered by latest amount of money,
+    item in returned list are [code, name, change, amount, price]
+    '''
     url = base64.b64decode('aHR0cHM6Ly82Mi5wdXNoMi5lYXN0bW9uZXkuY29tL2FwaS9xdC'+
         '9jbGlzdC9nZXQ/Y2I9alF1ZXJ5MTEyNDA4Mzc4MjAwMDc0NDQ0MzA5XzE2Mjc4MjQ2MDM'+
         '1NjImcG49MSZwej0yMDAwJnBvPTAmbnA9MSZ1dD1iZDFkOWRkYjA0MDg5NzAwY2Y5YzI3'+
@@ -324,25 +385,36 @@ def get_industry_stocks(bkid):
 
 
 def get_hk_stocks_ggt():
+    '''
+    Return sorted stock item list in GangGuTong, ordered by amount of money,
+    item in returned list are [code, name, change, amount, price]
+    '''
     a = _east_list_fmt('aHR0cHM6Ly8yLnB1c2gyLmVhc3Rtb25leS5jb20vYXBpL3F0L2NsaX'+
         'N0L2dldD9jYj1qUXVlcnkxMTI0MDI0MzYyMzA4OTA2NjE1MDgyXzE2MjgyNTg5MzEyMjQ'+
         'mcG49MSZwej0xMDAwJnBvPTAmbnA9MSZ1dD1iZDFkOWRkYjA0MDg5NzAwY2Y5YzI3ZjZm'+
         'NzQyNjI4MSZmbHR0PTImaW52dD0yJmZpZD1mNiZmcz1iOkRMTUswMTQ2LGI6RExNSzAxN'+
         'DQmZmllbGRzPWYzLGY2LGYxMixmMTQsZjIwJl89',
         'jQuery1124024362308906615082_1628258931224')
+    a = [ ['hk'+i[0], i[1], i[2], i[3], i[4]] for i in a]
     logging.debug('get hk stocks GangGuTong {}'.format(len(a)))
     return a
 
 
 def get_hk_stocks_hsi():
+    '''
+    Return sorted stock item list in HSI, ordered by amount of money,
+    item in returned list are [code, name, change, amount, price]
+    '''
     a = _east_list_fmt('aHR0cHM6Ly81Ni5wdXNoMi5lYXN0bW9uZXkuY29tL2FwaS9xdC9jbG'+
         'lzdC9nZXQ/Y2I9alF1ZXJ5MTEyNDA3ODg4ODY4NDU5NDc5NzkyXzE2MjgyNTk1NjQ2NzE'+
         'mcG49MSZwej0xMDAwJnBvPTEmbnA9MSZ1dD1iZDFkOWRkYjA0MDg5NzAwY2Y5YzI3ZjZm'+
         'NzQyNjI4MSZmbHR0PTImaW52dD0yJmZpZD1mNiZmcz1iOkRMTUswMTQxJmZpZWxkcz1mM'+
         'yxmNixmMTIsZjE0LGYyMCZfPQ==',
         'jQuery112407888868459479792_1628259564671')
+    a = [ ['hk'+i[0], i[1], i[2], i[3], i[4]] for i in a]
     logging.debug('get hk stocks HSI {}'.format(len(a)))
     return a
 
 
 
+
